///////////////////////////////////////////////////////////////////////////////
/*!\file    LINTests.c
\brief      Source zum Modul LINTests
\spezification  Dieses Source-File beinhaltet die komplette Umsetzung der 
            Funktionen des Moduls \ref LINTests sowie modulinterne Funktionen
            und Definitionen.
\ingroup    LINTests*/
///////////////////////////////////////////////////////////////////////////////



#include "..\..\LINTests\Header\LINTests.h"
#include "..\..\PXI-3070-LIN\Header\HPxi3070.h"



///////////////////////////////////////////////////////////////////////////////
/*!\brief   Senden/Beantworten eines LIN-Frame
\spezification  Die Funktion ermglicht es einen LIN-Frame zu senden bzw. zu 
            beantworten je nach angeschlossenem Prfling.
\param[in]  modulno ist die Nummer des Moduls, welches den Frame senden soll.
\param[in]  controller ist die Nummer des Controllers, der senden soll.
\param[in]  dut ist ein Pointer auf die Struktur vom Typ tDutData.
\param[in]  frame ist ein Pointer auf die Struktur vom Typ tLinFrame
\param[out] number ist die Anzahl empfangener und zurckgegebener Frames
\param[out] lindata ist ein Pointer auf die Struktur vom Typ TLINdata, welche 
            die empfangenen Daten der LIN-Karte enthlt.*/
///////////////////////////////////////////////////////////////////////////////
static void _SendFrame (int modulno, 
                        tControllerType controller, 
                        tDutData *dut, 
                        tLinFrame *frame, 
                        int *number, 
                        tLINdata lindata[MAXFROMDATAREAD])
{
    unsigned char data[1][8];   // Array zur bergabe an Funktionen fr die 
                                // LIN-Karte
    int i;                      // Zhlindex fr for-Schleife
    double delay = 100;         // Pause zwischen den Frames
    tBoolean wait;              // Flag zeigt an, ob die Schleife noch weiter
                                // warten soll

    // Auslesen der Monitordaten der LIN-Karte
    LIN_MonitorDataRead (modulno, controller, number, lindata);
    // Vorbereiten der Datenbergabe
    for (i = 0; i < frame->datalengthcount; i++)
        data[0][i] = frame->data[i];    
    // falls erforderlich, Datenbytes der LIN-Karte bermitteln
    if (frame->datalengthcount > 0)
        LIN_STableSet (modulno, controller, 0, 1, &frame->identifier, 
                       &frame->datalengthcount, data);
    // falls erforderlich, Sendetabelle der LIN-Karte einstellen und starten
    if (dut->type != LINTESTS_DUT_IS_LIN_MASTER)
    {
        LIN_MScheduleTableSet (modulno, controller, LIN_TABLE0, 
                               LIN_SET_TIME_ON, dut->baudrate, 
                               DEFAULT_SYNCHBREAK, DEFAULT_SYNCHDELIMITER,
                               DEFAULT_ARBITRATIONDELAY, 1, &frame->identifier,
                               &delay);
        LIN_MScheduleStart (modulno, controller, LIN_TABLE0);
    }
    
    // warte bis Frame erfolgreich empfangen wurde
    wait = TRUE;
    while (wait)
    {
        LIN_MonitorDataRead (modulno, controller, number, lindata);
        for (i = 0; i < *number; i++)
        {
            if (lindata[i].ident == frame->identifier)  
                if ( ((lindata[i].error == 0) && 
                      (frame->datalengthcount >= 0)) || 
                     (frame->datalengthcount < 0) ) 
                    wait = FALSE;
        }
    }
    
    // falls erforderlich Sende- und Antworttabelle lschen
    if (dut->type != LINTESTS_DUT_IS_LIN_MASTER)
    {
        LIN_MScheduleTableDelete (modulno, controller, LIN_TABLE0);
        if (frame->datalengthcount > 0)
            LIN_STableDelete (modulno, controller);
    }
    else
        LIN_STableDelete (modulno, controller);
}// _SendFrame



///////////////////////////////////////////////////////////////////////////////
/*!\brief   Sendet die LIN-Frames am Start
\spezification  Diese Funktion sendet die Frames auf dem LIN-Bus, die in der 
            Struktur tDutData.framesatstart abgelegt sind.
\param[in]  modulno ist die Nummer des Moduls, welches den Frame senden soll.
\param[in]  controller ist die Nummer des Controllers, der senden soll.
\param[in]  dut ist ein Pointer auf die Struktur vom Typ tDutData.*/
///////////////////////////////////////////////////////////////////////////////
static void _SendFramesAtStart (int modulno, 
                                tControllerType controller, 
                                tDutData *dut)
{
    int i;          // Zhlindex fr for-Schleife
    int number;     // Anzahl der zurckerhaltenen Daten der LIN-Karte
    tLINdata lindata[MAXFROMDATAREAD];  // Array fr die zurckerhaltenen
                                        // Daten der LIN-Karte
    
    // for-Schleife zum Senden smtlicher Frames
    for (i = 0; i < dut->numberframesatstart; i++)
    {
        _SendFrame (modulno, controller, dut, &dut->framesatstart[i], &number, 
                    lindata);
    }   
}// _SendFramesAtStart



