///////////////////////////////////////////////////////////////////////////////
/*!\file    HPxi3070.c
\brief      Source zum Modul LIN
\spezification  Dieses Source-File beinhaltet die komplette Umsetzung der 
            Funktionen des Moduls \ref LIN sowie modulinterne Funktionen
            und Definitionen.
\ingroup    LIN*/
///////////////////////////////////////////////////////////////////////////////



#include "..\Header\HPxi3070.h"



///////////////////////////////////////////////////////////////////////////////
/*!\brief   Setzen der Relais auf der LIN-Karte
\spezification  Diese Funktion bietet die Mglichkeit Relais auf der LIN-Karte
            zu setzen. In der Regel werden die Relais vom Controller 1 der 
            LIN-Karte angesteuert. Beim Setzen wird Relais 0-2 geffnet und
            Relais 3-5 und 7 geschlossen.
\param[in]  module ist die Modulnummer.
\param[in]  controller ist die Nummer des Controllers.
\param[in]  number ist die Anzahl Relais, die gesetzt werden soll
\param[in]  relay ist ein Feld, welches die Nummern der Relais enthlt, die 
            gesetzt werden sollen.
\sideeffect Mit Hilfe der Relais lassen sich Kanle z.B. miteinander 
            verschalten, wobei darauf geachtet werden sollte, dass sich stets
            nur ein aktiver Master am zusammengeschalteten Bus befindet, da
            es sonst mit groer Wahrscheinlichkeit zu massiven Strungen auf 
            dem Bus kommt.
\sa         LIN_MRelaisReset*/
///////////////////////////////////////////////////////////////////////////////
int LIN_MRelaisSet (int module, 
                    int controller, 
                    int number, 
                    int *relay)
{
    int i;  // Zhlindex fr for-Schleife
    // Struktur zum Informationsaustausch
    GPxi3070_StructWriteInstruction pWriteInstruction;
    GPxi3070_StructReadResponse     pReadResponse;

    // Lschen des Antwortstrings/Befehlstrings
    memset (&pReadResponse, 0, sizeof (pReadResponse));
    memset (pWriteInstruction.Instruction, 0, 
            sizeof (pWriteInstruction.Instruction));

    // Zusammensetzen des Befehls
    pWriteInstruction.Instruction[0] = 0x23;
    pWriteInstruction.Instruction[1] = module;        // Modulnummer
    pWriteInstruction.Instruction[2] = controller;    // Nummer des Controllers
    pWriteInstruction.Instruction[3] = 0x81;          // Befehlscode
    pWriteInstruction.Instruction[4] = number;        // Anzahl zu 
                                                      // schaltender Relais
    for (i=0;i<number;i++)
        pWriteInstruction.Instruction[5+i] = relay[i];// Relaisnummer 0..7 
    
    // Senden des Befehls
    #ifdef HW_PXI3070
        GPxi3070_WriteInstruction (module, controller, &pWriteInstruction);     
        GPxi3070_ReadResponse (module, controller, &pReadResponse);                 
    #endif  
    
    #ifdef HW_SERIAL
        if (LINComPort == 0) 
            _OpenCOM (USED_PORT);
        ComWrt (LINComPort, &pWriteInstruction.Instruction[0], 
                sizeof (pWriteInstruction.Instruction));
        //ComRd (LINComPort, &pReadResponse.Response[0], 
        //       sizeof (pReadResponse.Response));
    #endif  
    
    
    // falls Antwort gesendet wurde, dies im Errorstring vermerken
    if (pReadResponse.Response[0] != 0)
    {
        memcpy (ErrorMessage, pReadResponse.Response, sizeof (ErrorMessage));
        return (-1);
    }
    else
        return (0);
}// LIN_MRelaisSet           



///////////////////////////////////////////////////////////////////////////////
/*!\brief   Fllen der Antwortliste des Controllers
\spezification  Mit dieser Funktion kann die Antworttabelle gefllt werden. 
            Mehrmaliges Ausfhren der Funktion fhrt dazu, dass die vorhandene
            Liste um die neuen Objekte ergnzt wird. Bestehende Objekte
            gleichen Identifiers werden berschrieben.\n 
            Zu beachten ist, dass maximal 50 Bytes mit einem Befehl gesendet
            werden knnen, d.h. bergibt man eine Liste Identifier zu denen 
            8 Datenbytes gehren, so kann die Liste wegen des Befehlsoverhead
            maximal 4 Identifier inkl. Datenbytes enthalten.
\param[in]  module ist die Modulnummer.
\param[in]  controller ist die Nummer des Controllers.
\param[in]  interbyte ist die Interbyte-Zeit in 0,4s-Schritten
\param[in]  number ist die Anzahl der Objekte.
\param[in]  identifier ist ein Pointer auf ein Feld mit Identifiern
\param[in]  number_bytes ist ein Pointer auf ein Feld mit Angaben ber die
            Anzahl der Datenbytes.
\param[in]  data ist ein zweidimensionales Feld mit den Datenbytes fr die
            einzelnen Frames.
\sa         LIN_STableDelete*/
///////////////////////////////////////////////////////////////////////////////
int LIN_STableSet (int module, 
                   int controller, 
                   double interbyte, 
                   int number, 
                   int *identifier, 
                   int *number_bytes, 
                   unsigned char data[][8])
{
    int i;      // Zhlindex fr for-Schleife
    int j;      // Zhlindex fr for-Schleife
    int index;  // Zhlindex fr Position im Befehlsstring
    // Struktur zum Informationsaustausch
    GPxi3070_StructWriteInstruction pWriteInstruction;
    GPxi3070_StructReadResponse     pReadResponse;
            
    // Lschen des Antwortstrings/Befehlstrings
    memset (&pReadResponse, 0, sizeof (pReadResponse));
    memset (pWriteInstruction.Instruction, 0, 
            sizeof (pWriteInstruction.Instruction));

    // Zusammensetzen des Befehls
    pWriteInstruction.Instruction[0] = 0x23;
    pWriteInstruction.Instruction[1] = module;        // Modulnummer
    pWriteInstruction.Instruction[2] = controller;    // Nummer des Controllers
    pWriteInstruction.Instruction[3] = 0x23;          // Befehlscode
    pWriteInstruction.Instruction[4] = ((int)interbyte & 0xff00) >> 8;
    pWriteInstruction.Instruction[5] =  (int)interbyte & 0x00ff;
    
    pWriteInstruction.Instruction[6] = number;        // Anzahl der Frames

    index = 7;
    for (i=0;i<number;i++)
    {
        // Identifier einfgen
        pWriteInstruction.Instruction[index++]  = identifier[i];
        // Anzahl der Datenbytes einfgen
        pWriteInstruction.Instruction[index++]  = number_bytes[i];
        // Datenbytes einfgen
        for (j=0;j<number_bytes[i];j++)
            pWriteInstruction.Instruction[index++]= data[i][j]; // Datenbytes 
    }   
        
    // Senden des Befehls
    #ifdef HW_PXI3070
        GPxi3070_WriteInstruction (module, controller, &pWriteInstruction);     
        GPxi3070_ReadResponse (module, controller, &pReadResponse);                 
    #endif
    //Delay (0.010);

    #ifdef HW_SERIAL
        if (LINComPort == 0) 
            _OpenCOM (USED_PORT);
        ComWrt (LINComPort, &pWriteInstruction.Instruction[0], 
                sizeof (pWriteInstruction.Instruction));
        //ComRd (LINComPort, &pReadResponse.Response[0], 
        //       sizeof (pReadResponse.Response));
    #endif  

    // Es wird keine Besttigung erwartet
    // GPxi3070_ReadResponse (module, controller, &pReadResponse);                 
    // falls Antwort gesendet wurde, dies im Errorstring vermerken
    if (pReadResponse.Response[0] != 0)
    {
        memcpy (ErrorMessage, pReadResponse.Response, sizeof (ErrorMessage));
        return (-1);
    }
    else
        return (0);
}           
